#include "asyncorder.h"

void AsynCorder::process()
{
    m_rec = RecordInfo();
    m_camID = "none";
    m_state = STATE_UNKNOWN;
    m_lostFrameCounter = 0;
    m_flagGetTimeStampFirstPic = false;

    //here we connect start to a 10 min timer
    m_startDelayTimer = new QTimer(this);
    m_startDelayTimer->setInterval(VAR_TIME_DELAY_TO_WAIT_IN_mSEC); // 600000 millisec = 10 min

    //connect the start button with the timer
    QObject::connect(m_startDelayTimer, SIGNAL(timeout()), this, SLOT(startRecord()));


    // Start Vimba
    VmbErrorType err = m_ApiController.StartUp();
    qDebug()<< QString( "AsynchronousGrab (Qt version) Vimba Version V" )+ QString::fromStdString( m_ApiController.GetVersion() ) ;
    Log( "Starting Vimba", err );

    qDebug()<<"started recoding thread: "<<QThread::currentThreadId();

    if( VmbErrorSuccess == err )
    {
        // Connect new camera found event with event handler
        QObject::connect( m_ApiController.GetCameraObserver(), SIGNAL( CameraListChangedSignal(int) ), this, SLOT( OnCameraListChanged(int) ) );

        m_state = STATE_INIT_DONE;

        //we inform all followers
        emit stateChange(m_state);

        // Initially get all connected cameras
        //UpdateCameraListBox();
        //        std::stringstream strMsg;
        //        strMsg << "Cameras found..." << m_cameras.size();
        //        Log(strMsg.str() );
    }

    //this->exec();
}

QString AsynCorder::camID() const
{
    return m_camID;
}

void AsynCorder::setCamID(const QString &camID)
{
    m_camID = camID;
}

bool AsynCorder::chooseCamToRecord(QString camID)
{
    QStringList list = this->getCamList();
    if(list.contains(camID,Qt::CaseInsensitive))
    {
        m_camID = camID;
        qDebug()<<QThread::currentThreadId()<<" will use cam: "<< camID;

        return true;
    }
    else
        return false;
}

cv::Mat AsynCorder::qImageToMatRef(QImage &img, int format)
{
    return cv::Mat(img.height(), img.width(),
                   format, img.bits(), img.bytesPerLine());
}

cv::Mat AsynCorder::qImageToMatDeepCpy(QImage const &img, int format)
{
    //maybe also swap the bites already ?
    return cv::Mat(img.height(), img.width(), format,
                   const_cast<uchar*>(img.bits()),
                   img.bytesPerLine()).clone();
}

// This event handler is triggered through a Qt signal posted by the frame observer
void AsynCorder::OnFrameReady( int status )
{
    QTime myTimer;
    myTimer.start();
    quint64 tMsec = 0;
    quint64 tSec = 0;

    if( m_state == STATE_RECORDING )
    {
        // Pick up frame
        FramePtr pFrame = m_ApiController.GetFrame();
        if( SP_ISNULL( pFrame ) )
        {
            Log("frame pointer is NULL, late frame ready message");
            return;
        }
        // See if it is not corrupt
        if( VmbFrameStatusComplete == status )
        {
            VmbUchar_t *pBuffer;
            VmbUint64_t timestamp = 0 ;
            VmbErrorType err = SP_ACCESS( pFrame )->GetImage( pBuffer );
            if( VmbErrorSuccess == err )
            {
                VmbUint32_t nSize;
                //create a time stamp

                //                QTime thisTime = QTime::currentTime();
                //                QDateTime thisDateTime =  QDateTime::currentDateTime();

                //qDebug()<<"grab picture at:"<<thisDateTime.toTime_t()<<" ; "<< thisTime.msec();
                //qDebug()<< "TimeObject::setToCurrentTime() time for call QDateTime::currentDateTime(); "<< myTimer.elapsed();
                //qDebug()<< "TimeObject::currentDateTime() time for call "<<thisDateTime.toString("dd.MM.yyyy") << " seconds" <<  thisDateTime.toTime_t();

                //cv::Mat image; //this will be our shared pointer class

                //get the internal timestamp
                err = SP_ACCESS( pFrame )->GetTimestamp(timestamp);
                if( VmbErrorSuccess == err )
                {
                    //calc real world time
                    //double fraction = (double) 1.0 /(double) VAR_GevTimestampTickFrequency_in_HZ;
                    // formular from doc: Image Timestamp on Allied Vision GigE Cameras
                    double t = (double)( (double) timestamp / (double)VAR_GevTimestampTickFrequency_in_HZ);
                    //double t2 = timestamp * fraction;
                    //calc fractional parts ?
                    //TODO check     is this compiler and plattform independ ?
                    tMsec = (int)( ( t- ((long) t) ) * 1000); //maybe use round ?
                    tSec = (int) t;

                    //qDebug()<<QThread::currentThreadId()<<m_imgCounter<<" ts:"<<timestamp <<"\t"<<t << "s:"<<tSec<<"\tms:"<< tMsec;

                    //if its the first frame per record, we get the time on both systems
                    if(m_flagGetTimeStampFirstPic)
                    {
                        //we calc the offset
                        //get now
                        QTime thisTime = QTime::currentTime();
                        QDateTime thisDateTime =  QDateTime::currentDateTime();
                        quint64 timeS = thisDateTime.toTime_t();
                        quint64 timeMS = thisTime.msec();

                        m_startMoment.setTStartServerS(timeS);
                        m_startMoment.setTStartServerMS(timeMS);
                        m_startMoment.setTStartCamS(tSec);
                        m_startMoment.setTStartCamMS(tMsec);

                        m_flagGetTimeStampFirstPic = false;

                    }

                }
                else
                {
                    std::string errorMsg =  m_ApiController.ErrorCodeToMessage( err );
                    qWarning()<<"error during read timestamp"<< QString::fromStdString(errorMsg);
                }

                err = SP_ACCESS( pFrame )->GetImageSize( nSize );
                if( VmbErrorSuccess == err )
                {
                    VmbPixelFormatType ePixelFormat = m_ApiController.GetPixelFormat();
                    //qDebug()<<"pixel format:" << (int) ePixelFormat;
                    if( ! m_Image.isNull() )
                    {
                        // Copy it
                        // We need that because Qt might repaint the view after we have released the frame already
                        //if( ui.m_ColorProcessingCheckBox->checkState()==  Qt::Checked )
                        //                        {
                        //                            static const VmbFloat_t Matrix[] = {    8.0f, 0.1f, 0.1f, // this matrix just makes a quick color to mono conversion
                        //                                                                    0.1f, 0.8f, 0.1f,
                        //                                                                    0.0f, 0.0f, 1.0f };
                        //                            if( VmbErrorSuccess != CopyToImage( pBuffer,ePixelFormat, m_Image, Matrix ) )
                        //                            {
                        //                                //ui.m_ColorProcessingCheckBox->setChecked( false );
                        //                            }
                        //                        }
                        //                        else
                        {
                            //TODO write function to write to opencv image
                            //https://stackoverflow.com/questions/14727267/opencv-read-jpeg-image-from-buffer
                            //https://asmaloney.com/2013/11/code/converting-between-cvmat-and-qimage-or-qpixmap/

                            CopyToImage( pBuffer,ePixelFormat, m_Image );

                            //we copied all we rename the image buffer to opencv map
                            //one way could be to set the ref

                            //we make a deep copy
                            //a test emphases, this operation costs nearly nothing than ram
                            cv::Mat image = qImageToMatDeepCpy(m_Image,CV_8UC3);

                            //if we overhand image, everthing is a reference
                            //unles we call clone http://answers.opencv.org/question/14798/what-is-the-best-practise-for-passing-cvmats-around/
                            ImageProcessTask *myTask = new ImageProcessTask();
                            myTask->setImage(image);
                            //set the time time when the number and increment
                            myTask->setSecond(tSec);
                            myTask->setMilliSecond(tMsec);

                            myTask->setStartMoment(m_startMoment);
                            myTask->setImageNumber(m_imgCounter++);
                            //myTask->setDateTime(thisTime,thisDateTime);
                            //TODO: make it beautyfull
                            //TODO: we move this
                            myTask->setPath(UtilFactoryDataFileName::getPathStringOfRecord(m_rec));
                            //old version
                            // QString("/tmp/test/") + QString(m_camID.replace("DEV_","")) + QString("/pics/"));

                            // QThreadPool takes ownership and deletes 'hello' automatically
                            QThreadPool::globalInstance()->start(myTask);

                        }

                    }
                }
            }
        }
        else
        {
            // If we receive an incomplete image we do nothing but logging
            //Log( "Failure in receiving image  ! ", VmbErrorOther );
            m_lostFrameCounter++;
        }

        // And queue it to continue streaming
        m_ApiController.QueueFrame( pFrame );
    }

    int nMilliseconds = myTimer.elapsed();

    if((m_imgCounter%100)==0)
        qDebug()<<QThread::currentThreadId()<<"|"<<m_camID<<"|"<<m_imgCounter<<"frame\t-\t"<<nMilliseconds<<"ms\t"<< (double)( (double) 1000 / (double)nMilliseconds) << " fps - #lost frames: "<< m_lostFrameCounter ;

}

void AsynCorder::startDelay()
{
    qDebug()<<"start "<< m_camID << " called";

    m_state = STATE_START_DELAY;

    //we inform all followers
    emit stateChange(m_state);

    m_startDelayTimer->start();

}

void AsynCorder::startRecord()
{

    qDebug()<<"start "<< m_camID << " called";

    VmbErrorType err;

    if(m_state==STATE_INIT_DONE||m_state==STATE_WAITING||m_state==STATE_START_DELAY)
    {
        //we check if an id was found
        if(QString::compare(m_camID,"none",Qt::CaseInsensitive)!=0)
        {
            //counter
            m_imgCounter = 0;

            //we stop the delay timer
            m_startDelayTimer->stop();

            // Start acquisition
            std::string camID = m_camID.toStdString();
            err = m_ApiController.StartContinuousImageAcquisition( camID );
            // if the start was good
            if ( VmbErrorSuccess == err )
            {
                m_Image = QImage( m_ApiController.GetWidth(),
                                  m_ApiController.GetHeight(),
                                  QImage::Format_RGB888 );

                QObject::connect( m_ApiController.GetFrameObserver(), SIGNAL( FrameReceivedSignal(int) ), this, SLOT( OnFrameReady(int) ) );

                m_state = STATE_RECORDING;

                //we inform all followers
                emit stateChange(m_state);

                //here we set a timer for
                //we use a automatic stop to prevent overflow of the record storage buffer
                unsigned int msWait = 3660000; // 61 min  in ms
                QTimer::singleShot(msWait, this, SLOT(stopRecord()));

                //we construct a new record info **********************
                m_rec = RecordInfo();

                //generate the needed uid, without a conflict
                int tmpUID = -1;
                do{
                    tmpUID = UtilFactoryDataFileName::getUID();
                }while(ConfigManager::recList->existUIDinList(tmpUID));

                m_rec.setUID(tmpUID);
                m_rec.setCamID(m_camID);

                //TODO move this to record class or util
                //we set the start time
                QTime thisTime = QTime::currentTime();
                QDateTime thisDateTime =  QDateTime::currentDateTime();

                //now trigger flag for getting the first timeStamp
                m_flagGetTimeStampFirstPic = true;

                //set rec start time
                m_rec.setTimeStartSecUnix(thisDateTime.toTime_t());
                m_rec.setTimeStartMS(thisTime.msec());
                m_rec.setState(RecordInfo::STATE_RECORDING_AT_THE_MOMENT);

                //we add this to the conf mananger
                bool succ = ConfigManager::instance()->reloadRecordList();

                if(!succ)
                    qWarning()<<"error load record list from file";

                //we add this to the conf mananger
                ConfigManager::instance()->recList->appendRecord(m_rec);
                //we save the changes
                if(!ConfigManager::instance()->writeRecordList())
                    qWarning()<<"error during write record info to file";
            }//end if start was fine
            else
            {
                qWarning()<<"error during start the recording on cam: "<< m_camID;
                std::string strMsg = m_ApiController.ErrorCodeToMessage( err );
                qWarning()<< QString::fromStdString( strMsg ) ;

                //                qWarning()<<"try another time to stopp cam";
                //                //fake the state
                //                m_state = STATE_RECORDING;
                //                stopRecord();

            }
        }
        else
            qWarning()<<"error no camID found";
    }
    else
        qWarning()<<"tried to start recording but wrong internal state";
}

QString AsynCorder::stateToString(STATE s)
{

    //good solution from stackoverflow
    //@see https://stackoverflow.com/questions/28828957/enum-to-string-in-modern-c11-c14-c17-and-future-c20

    const QMap<AsynCorder::STATE,QString> MyEnumStrings {
        { AsynCorder::STATE_UNKNOWN, QString("unkown") },
        { AsynCorder::STATE_INIT_DONE, QString("init done") },
        { AsynCorder::STATE_START_DELAY, QString("delay") },
        { AsynCorder::STATE_RECORDING, QString("recording") },
        { AsynCorder::STATE_WAITING, QString("waiting") },

        { AsynCorder::STATE_ERROR_CRITICAL, QString("error") }
    };
    auto   it  = MyEnumStrings.find(s);
    return it == MyEnumStrings.end() ? "Out of range" : it.value();

}

void AsynCorder::stopRecord()
{
    qDebug()<<"stop "<< m_camID << " called";

    VmbErrorType err;

    m_startMoment = StartMoment();

    if(m_state==STATE_RECORDING)
    {

        // Stop acquisition
        err = m_ApiController.StopContinuousImageAcquisition();

        if(VmbErrorSuccess != err )
            qWarning()<<"StopContinuousImageAcquisition produced error: "<< err;

        m_state = STATE_WAITING;

        //we inform all followers
        emit stateChange(m_state);

        // Clear all frames that we have not picked up so far
        m_ApiController.ClearFrameQueue();
        m_Image = QImage();
        //Log( "Stopping Acquisition", err );

        //we update the old record info**************
        //we set the stop time
        QTime thisTime = QTime::currentTime();
        QDateTime thisDateTime =  QDateTime::currentDateTime();

        m_rec.setTimeStoppSecUnix(thisDateTime.toTime_t());
        m_rec.setTimeStoppMS(thisTime.msec());
        m_rec.setState(RecordInfo::STATE_RECORDED_WITH_SUCCESS);

        //we also will save stats
        m_rec.setAmountFramesRecorded(m_imgCounter-1);
        m_rec.setAmountLostFrames(m_lostFrameCounter);

        //we reset also the counter
        m_imgCounter =0;
        m_lostFrameCounter = 0;


        //we add this to the conf mananger
        bool succ = ConfigManager::instance()->reloadRecordList();

        if(!succ)
            qWarning()<<"error load record list from file";

        succ = ConfigManager::instance()->recList->updateRecord(m_rec);

        if(!succ)
            qWarning()<<"error during update the record info";

        //TODO there is a small change for inconsitenc read and write for this file

        //we save the changes
        if(!ConfigManager::instance()->writeRecordList())
            qWarning()<<"error during write record info to file";


    }
    else
        qWarning()<<"try stop recording but wrong internal state";
}

//void AsynCorder::OnBnClickedButtonStartstop()
//{
//    qDebug()<<"start | stop called";

//    VmbErrorType err;
//    //get the id
//    //int nRow = ui.m_ListBoxCameras->currentRow();

//    //TODO implement a timer reset -> to prevent overflow of timestamp!

//    //if( -1 < nRow )
//    //we check if we set a camID
//    if(QString::compare(m_camID,"none",Qt::CaseInsensitive)!=0)
//    {
//        if( false == m_bIsStreaming ) //if we not recording
//        {
//            //counter
//            m_imgCounter = 0;

//            // Start acquisition
//            std::string camID = m_camID.toStdString();
//            err = m_ApiController.StartContinuousImageAcquisition( camID );
//            // if the start was good
//            if ( VmbErrorSuccess == err )
//            {
//                m_Image = QImage( m_ApiController.GetWidth(),
//                                  m_ApiController.GetHeight(),
//                                  QImage::Format_RGB888 );

//                QObject::connect( m_ApiController.GetFrameObserver(), SIGNAL( FrameReceivedSignal(int) ), this, SLOT( OnFrameReady(int) ) );

//                //set the flag that we are recording now
//                m_bIsStreaming = true ;


//                //we construct a new record info **********************
//                m_rec = RecordInfo();

//                //generate the needed uid, without a conflict
//                int tmpUID = -1;
//                do{
//                    tmpUID = UtilFactoryDataFileName::getUID();
//                }while(ConfigManager::recList->existUIDinList(tmpUID));

//                m_rec.setUID(tmpUID);
//                m_rec.setCamID(m_camID);

//                //TODO move this to record class or util
//                //we set the start time
//                QTime thisTime = QTime::currentTime();
//                QDateTime thisDateTime =  QDateTime::currentDateTime();

//                m_rec.setTimeStartMS(thisDateTime.toTime_t());
//                m_rec.setTimeStartSecUnix(thisTime.msec());
//                m_rec.setState(RecordInfo::STATE_RECORDING_AT_THE_MOMENT);

//                //we add this to the conf mananger
//                if(!ConfigManager::instance()->recList->appendRecord(m_rec))
//                     qWarning()<<"error during append record info to list";
//                //we save the changes
//                if(!ConfigManager::instance()->writeRecordList())
//                    qWarning()<<"error during write record info to file";
//            }//end if start was fine

//        }
//        else
//        {
//            m_bIsStreaming = false;
//            // Stop acquisition
//            err = m_ApiController.StopContinuousImageAcquisition();
//            // Clear all frames that we have not picked up so far
//            m_ApiController.ClearFrameQueue();
//            m_Image = QImage();
//            Log( "Stopping Acquisition", err );

//            //we update the old record info**************
//            //we set the stop time
//            QTime thisTime = QTime::currentTime();
//            QDateTime thisDateTime =  QDateTime::currentDateTime();

//            m_rec.setTimeStoppMS(thisDateTime.toTime_t());
//            m_rec.setTimeStoppSecUnix(thisTime.msec());
//            m_rec.setState(RecordInfo::STATE_RECORDED_WITH_SUCCESS);

//            //we also will save stats
//            m_rec.setAmountFramesRecorded(m_imgCounter-1);
//            m_rec.setAmountLostFrames(m_lostFrameCounter);

//            //we reset also the counter
//            m_imgCounter =0;
//            m_lostFrameCounter = 0;


//            //we add this to the conf mananger
//            bool succ = ConfigManager::instance()->recList->updateRecord(m_rec);

//            if(!succ)
//                qWarning()<<"error during update the record info";

//            //we save the changes
//            if(!ConfigManager::instance()->writeRecordList())
//                qWarning()<<"error during write record info to file";


//        }

//        //we inform about the state have the state

//        if( false == m_bIsStreaming )
//        {
//            //ui.m_ButtonStartStop->setText( QString( "Start Image Acquisition" ) );
//            qDebug()<<"stopped   on :"<< m_camID;
//        }
//        else
//        {
//            //            ui.m_ButtonStartStop->setText( QString( "Stop Image Acquisition" ) );
//            qDebug()<<"started on :"<< m_camID;
//        }
//    }//if we set a good cam id
//}


// This event handler is triggered through a Qt signal posted by the camera observer
void AsynCorder::OnCameraListChanged( int reason )
{
    //bool bUpdateList = false;

    // We only react on new cameras being found and known cameras being unplugged
    if( AVT::VmbAPI::UpdateTriggerPluggedIn == reason )
    {
        Log( "Camera list changed. A new camera was discovered by Vimba." );
        //        bUpdateList = true;
    }
    else if( AVT::VmbAPI::UpdateTriggerPluggedOut == reason )
    {
        //        Log( "Camera list changed. A camera was disconnected from Vimba." );
        //        if( true == m_bIsStreaming )
        //        {
        //            OnBnClickedButtonStartstop();
        //        }
        //        //bUpdateList = true;
    }


    //TODO update global

}

// Copies the content of a byte buffer to a Qt image with respect to the image's alignment
VmbErrorType AsynCorder::CopyToImage( VmbUchar_t *pInBuffer, VmbPixelFormat_t ePixelFormat, QImage &pOutImage, const float *Matrix /*= NULL */ )
{
    const int           nHeight = m_ApiController.GetHeight();
    const int           nWidth  = m_ApiController.GetWidth();

    VmbImage            SourceImage,DestImage;
    VmbError_t          Result;
    SourceImage.Size    = sizeof( SourceImage );
    DestImage.Size      = sizeof( DestImage );

    Result = VmbSetImageInfoFromPixelFormat( ePixelFormat, nWidth, nHeight, & SourceImage );
    if( VmbErrorSuccess != Result )
    {
        Log( "Could not set source image info", static_cast<VmbErrorType>( Result ) );
        return static_cast<VmbErrorType>( Result );
    }
    QString             OutputFormat;
    const int           bytes_per_line = pOutImage.bytesPerLine();
    switch( pOutImage.format() )
    {
    default:
        Log( "unknown output format",VmbErrorBadParameter );
        return VmbErrorBadParameter;
    case QImage::Format_RGB888:
        if( nWidth*3 != bytes_per_line )
        {
            Log( "image transform does not support stride",VmbErrorWrongType );
            return VmbErrorWrongType;
        }
        OutputFormat = "RGB24";
        break;
    }
    Result = VmbSetImageInfoFromString( OutputFormat.toStdString().c_str(), OutputFormat.length(),nWidth,nHeight, &DestImage );
    if( VmbErrorSuccess != Result )
    {
        Log( "could not set output image info",static_cast<VmbErrorType>( Result ) );
        return static_cast<VmbErrorType>( Result );
    }
    SourceImage.Data    = pInBuffer;
    DestImage.Data      = pOutImage.bits();
    // do color processing?
    if( NULL != Matrix )
    {
        //this branch is not used
        VmbTransformInfo TransformParameter;
        Result = VmbSetColorCorrectionMatrix3x3( Matrix, &TransformParameter );
        if( VmbErrorSuccess == Result )
        {
            Result = VmbImageTransform( &SourceImage, &DestImage, &TransformParameter,1 );
        }
        else
        {
            Log( "could not set matrix t o transform info ", static_cast<VmbErrorType>( Result ) );
            return static_cast<VmbErrorType>( Result );
        }
    }
    else
    {
        Result = VmbImageTransform( &SourceImage, &DestImage,NULL,0 );
    }
    if( VmbErrorSuccess != Result )
    {
        Log( "could not transform image", static_cast<VmbErrorType>( Result ) );
        return static_cast<VmbErrorType>( Result );
    }
    return static_cast<VmbErrorType>( Result );
}

QStringList AsynCorder::getCamList()
{
    QStringList list;
    //load all cams we could find
    // Get all cameras currently connected to Vimba
    CameraPtrVector cameras = m_ApiController.GetCameraList();

    // And query the camera details again
    for( CameraPtrVector::const_iterator iter = cameras.begin();
         cameras.end() != iter;
         ++iter )
    {
        std::string strCameraName;
        std::string strCameraID;
        if( VmbErrorSuccess != (*iter)->GetName( strCameraName ) )
        {
            strCameraName = "[NoName]";
        }
        // If for any reason we cannot get the ID of a camera we skip it
        if( VmbErrorSuccess == (*iter)->GetID( strCameraID ) )
        {
            list.append(QString::fromStdString(strCameraID ));
            //m_cameras.push_back( strCameraID );
        }
    }

    return list;
}

// Prints out some logging
void AsynCorder::Log( std::string strMsg, VmbErrorType eErr )
{
    strMsg += "..." + m_ApiController.ErrorCodeToMessage( eErr );
    qDebug()<< QString::fromStdString( strMsg ) ;
}
// Prints out some logging without error
void AsynCorder::Log( std::string strMsg)
{
    //ui.m_ListLog->insertItem( 0, QString::fromStdString( strMsg ) );
    qDebug()<< QString::fromStdString( strMsg ) ;
}
