#ifndef ASYNCORDER_H
#define ASYNCORDER_H

//own
#include "imageprocesstask.h"
#include "startmoment.h"

//vimba interface
#include <ApiController.h>
#include "VimbaImageTransform/Include/VmbTransform.h"

//define two modes
#define MODE_A_LOG
//#define MODE_A_SHOW

//opencv
#include "opencv2/core/core.hpp"
#include "opencv2/opencv.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"



//qt
#include <QRunnable>
#include <QObject>
#include <QThread>
#include <QDebug>
#include <QTime>
#include <QTimer>
#include <QDir>
#include <QDateTime>
#include <QImageWriter>
#include <QImage>
#include <QQueue>
#include <QList>
#include <QMap>


//std
#include <sstream>

#define NUM_COLORS 3
#define BIT_DEPTH 8

//hard coded feature of the cam, read the feature value from vimba
#define VAR_GevTimestampTickFrequency_in_HZ 1000000000
#define VAR_TIME_DELAY_TO_WAIT_IN_mSEC 600000 // 600000 milli seconds = 10 min

using AVT::VmbAPI::Examples::ApiController;
using AVT::VmbAPI::FramePtr;
using AVT::VmbAPI::CameraPtrVector;

//! thread implementation for async frame capturing
class AsynCorder : public QObject
{
    Q_OBJECT

public:

    enum STATE
    {
        STATE_UNKNOWN,
        STATE_INIT_DONE,
        STATE_WAITING,
        STATE_START_DELAY,
        STATE_RECORDING,
        STATE_ERROR_CRITICAL
    };


    QString camID() const;
    void setCamID(const QString &camID);

    //! returns the list of the specific cam id which could be found from the interface
    QStringList getCamList();

    //! will set the cam id to be recorded
    //! double check if the id is present
    bool chooseCamToRecord(QString camID);

    static QString stateToString(STATE s);

public slots:


    //implement the run method
    //! this is doing work as a constructor why ?
    void process();

//    //! The event handler for starting / stopping acquisition
//    void OnBnClickedButtonStartstop();

    //! we
    void startRecord();

    void startDelay();

    //!
    void stopRecord();

signals:

    //! inform about a state change
    void stateChange(AsynCorder::STATE s);

private:


    STATE m_state;

    RecordInfo m_rec;

//    STATE m_state;
    //QQueue<double> m_fpsHistory;
    double m_fps;

    quint64 m_lostFrameCounter;

    QTimer* m_startDelayTimer;

    ApiController m_ApiController;
    // A list of known camera IDs
    //std::vector<std::string> m_cameras;
    QString m_camID;
    // Are we streaming?
    bool m_bIsStreaming;
    // Our Qt image to display
    QImage m_Image;
    bool m_flagGetTimeStampFirstPic;
    StartMoment m_startMoment;//TODO reduce doubled info at record info

    //image counter
    quint64 m_imgCounter;
    // Prints out some logging
    void Log( std::string strMsg, VmbErrorType eErr );
    void Log( std::string strMsg);
    // Copies the content of a byte buffer to a MFC image
    VmbErrorType CopyToImage( VmbUchar_t *pInBuffer, VmbPixelFormat_t ePixelFormat, QImage &pOutImage, const float *Matrix = NULL );

    //! @see https://qtandopencv.blogspot.de/2013/08/how-to-convert-between-cvmat-and-qimage.html
    //! will return ref to mat
    cv::Mat qImageToMatRef(QImage &img, int format);

    //! will make a deep copy
    cv::Mat qImageToMatDeepCpy(QImage const &img, int format);

private slots:

    // The event handler for the frame received event
    void OnFrameReady( int status );
    // The event handler for the camera list changed event
    void OnCameraListChanged( int reason );
};

#endif // ASYNCORDER_H
