#include "config.h"


Config::Config()
{
    m_storagePath = ".";
    //m_fileLenseCorrection = "camera.yml";
    m_globalRecordUIDListFile ="recordlist.json";
    m_minDiskSpaceNeeded = -1;
    m_recordDurationInSec = 3600; // 1 h
}


QString Config::toString()
{
    QString s = "";

    QJsonObject recJO;
    this->write(recJO);
    QJsonDocument saveDoc(recJO);
    s.append(saveDoc.toJson());

    return s;
}

void Config::read(const QJsonObject &json)
{
    if (json.contains("path") && json["path"].isString())
        m_storagePath = json["path"].toString();

    if (json.contains("globalRecordUIDListFile") && json["globalRecordUIDListFile"].isString())
        m_globalRecordUIDListFile = json["globalRecordUIDListFile"].toString();
    //    if (json.contains("fileLenseCorrection") && json["fileLenseCorrection"].isString())
    //        m_fileLenseCorrection = json["fileLenseCorrection"].toString();

    //foo
    if (json.contains("camlist") && json["camlist"].isArray()) {
        QJsonArray camArray = json["camlist"].toArray();
        m_camList.clear();
        m_camList.reserve(camArray.size());
        for (int camListIndex = 0; camListIndex < camArray.size(); ++camListIndex) {
            QJsonObject npcObject = camArray[camListIndex].toObject();
            ConfCamera camTmp;
            camTmp.read(npcObject);
            m_camList.append(camTmp);
        }
    }

    m_minDiskSpaceNeeded = getValueInt(json, "minSpaceInGB");
    if(m_minDiskSpaceNeeded==-1)
        qWarning()<<"error during parse config file ";

    m_recordDurationInSec = getValueInt(json, "recordDurationInSec");
    if(m_recordDurationInSec==-1)
        qWarning()<<"error during parse config file ";

}

int Config::getValueInt(const QJsonObject &json, QString jsonPattern)
{
    int value = -1;

    if (json.contains(jsonPattern) && json[jsonPattern].isDouble())
        value = json[jsonPattern].toInt();

    return value;
}

void Config::write(QJsonObject &json) const
{
    json["path"] = m_storagePath;
    //json["fileLenseCorrection"] = m_fileLenseCorrection;
    json["globalRecordUIDListFile"] = m_globalRecordUIDListFile;
    json["minSpaceInGB"] = m_minDiskSpaceNeeded;
    json["recordDurationInSec"] = m_recordDurationInSec;

    QJsonArray camArray;
    foreach (const ConfCamera camTmp, m_camList) {
        QJsonObject camObject;
        camTmp.write(camObject);
        camArray.append(camObject);
    }
    json["camlist"] = camArray;

}

QString Config::storagePath() const
{
    return m_storagePath;
}

void Config::setStoragePath(const QString &storagePath)
{
    m_storagePath = storagePath;
}

int Config::minDiskSpaceNeeded() const
{
    return m_minDiskSpaceNeeded;
}

void Config::setMinDiskSpaceNeeded(int minDiskSpaceNeeded)
{
    m_minDiskSpaceNeeded = minDiskSpaceNeeded;
}


//QString Config::getFileLenseCorrection() const
//{
//    return m_fileLenseCorrection;
//}

//void Config::setFileLenseCorrection(const QString &fileLenseCorrection)
//{
//    m_fileLenseCorrection = fileLenseCorrection;
//}

QList<ConfCamera> Config::getCamList() const
{
    return m_camList;
}

void Config::setCamList(const QList<ConfCamera> &camList)
{
    m_camList = camList;
}

QString Config::getGlobalRecordUIDListFile() const
{
    return m_globalRecordUIDListFile;
}

void Config::setGlobalRecordUIDListFile(const QString &globalRecordUIDListFile)
{
    m_globalRecordUIDListFile = globalRecordUIDListFile;
}

int Config::getRecordDurationInSec() const
{
    return m_recordDurationInSec;
}

void Config::setRecordDurationInSec(int recordDurationInSec)
{
    m_recordDurationInSec = recordDurationInSec;
}


