#include "configmanager.h"


//declare static memembers ************

//must be for declerate the singelton
ConfigManager* ConfigManager::m_instance = 0;


Config* ConfigManager::conf = nullptr;
RecordList* ConfigManager::recList = nullptr;

bool ConfigManager::m_loadedDistortionMatrixSet = false;
cv::Mat ConfigManager::m_intrinsicsCameraMatrix = cv::Mat();
cv::Mat ConfigManager::m_distortionCoeff = cv::Mat();

//methods*************************************
ConfigManager* ConfigManager::instance()
{

    static QMutex mutex;
    if (!m_instance)
    {
        mutex.lock();

        if (!m_instance)
            m_instance = new ConfigManager();

        mutex.unlock();
    }

    return m_instance;
}


void ConfigManager::drop()
{
    static QMutex mutex;
    mutex.lock();
    delete m_instance;
    m_instance = 0;
    mutex.unlock();
}

ConfigManager::ConfigManager()
{
    //m_frameCounter = 0;
}

//******************************** singelton stuff ***end

bool ConfigManager::writeConfig(QString path,Config config)
{
    QString pathToFile(path +  QStringLiteral("config.json"));
    QFile saveFile(pathToFile);

    if (!saveFile.open(QIODevice::WriteOnly)) {
        qWarning()<<"IO-error: Couldn't open file on path "<< path;
        return false;
    }

    QJsonObject configJO;
    config.write(configJO);

    //TODO copy the conf object

    QJsonDocument saveDoc(configJO);
    qDebug()<<saveDoc.toJson() <<" to file: "<< pathToFile;
    if(saveFile.write(saveDoc.toJson())==-1)
        qWarning()<<"error during write config file to:" <<pathToFile;


    saveFile.close();

    return true;
}


bool ConfigManager::writeRecordList()
{
    return writeRecordList(conf->storagePath(),(*recList));
}

bool ConfigManager::writeRecordList(QString path,RecordList &recList)
{
    QString pathToFile(path +  QStringLiteral("recordlist.json"));

    //qDebug()<<"write record list to :"<< pathToFile;

    //if not exist create the folder
    UtilFactoryDataFileName::createDir(path);


    QFile saveFile(pathToFile);

    if (!saveFile.open(QIODevice::WriteOnly)) {
        qWarning()<<"IO-error: Couldn't open file on path "<< path;
        return false;
    }

    QJsonObject recJO;
    recList.write(recJO);

    //TODO copy the conf object

    QJsonDocument saveDoc(recJO);
    //qDebug()<<saveDoc.toJson() <<" to file: "<< pathToFile;
    if(saveFile.write(saveDoc.toJson())==-1)
        qWarning()<<"error during write config file to:" <<pathToFile;


    saveFile.close();

    qDebug()<<"done";

    return true;
}


bool ConfigManager::loadConfig()
{
    return loadConfig(UtilFactoryDataFileName::getDefaultPathConfigFile());
}

bool ConfigManager::loadConfig(QString path)
{
    //load the global config file **************************
    bool parseSuccess = false;

    QString pathToConfigFile(path + QStringLiteral("config.json"));
    QJsonDocument loadDocConf = loadJsonDocFromFile(pathToConfigFile,parseSuccess);

    if(parseSuccess)
    {
        if(conf==nullptr)
            conf = new Config();
        //we read the content
        conf->read( loadDocConf.object()   );
    }
    else
    {

        qWarning()<<"no load of config !! will abort further conf reading";
        return false;
    }

    //we also load the record list
    return reloadRecordList();

}

bool ConfigManager::reloadRecordList()
{

    if(conf==nullptr)
    {
        qWarning()<<"error during load config object";
        return false;

    }

    //we also try to load our global record list
    bool  parseSuccess = false;
    QString filePathToRecordList(conf->storagePath() + conf->getGlobalRecordUIDListFile());
    QJsonDocument loadDocRercordFile = loadJsonDocFromFile(filePathToRecordList,parseSuccess);

    //we construct a new RecordList in any case
    if(recList==nullptr)
        recList = new RecordList();

    if(parseSuccess)
    {
        //we read the content, we already have
        recList->read( loadDocRercordFile.object()   );
    }
    else
    {
        qWarning()<<"NO record file found:  "<< filePathToRecordList <<" will create one";
        //we construct a new empty file
        writeRecordList();
    }


    return true;
}

QJsonDocument ConfigManager::loadJsonDocFromFile(QString file, bool &success)
{
    success = false;
    QJsonDocument loadDoc;

    //check if file exists
    if(UtilFactoryDataFileName::fileExists(file))
    {

        //check if file readable
        QFile loadFile(file);
        if (!loadFile.open(QIODevice::ReadOnly)) {
            qWarning()<<"IO-error: Couldn't open file"<< file<< " will abort";
        }

        QByteArray saveData = loadFile.readAll();

        //try to parse as json
        QJsonParseError errObj;
        loadDoc =  QJsonDocument::fromJson(saveData,&errObj);

        if(errObj.error == QJsonParseError::NoError)
            success = true;
        else
            qWarning()<<"error during parse file: " << file<<" with err: "<< errObj.errorString();
    }
    else
        qWarning()<<"file not found: "<<file;

    return loadDoc;
}

Config* ConfigManager::getConfig()
{
    return conf;
}
