#ifndef CONFIGMANAGER_H
#define CONFIGMANAGER_H

//qt
#include <QJsonObject>
#include <QJsonDocument>
#include <QFile>
#include <QMutex>

//opencv
#include "opencv2/opencv.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"

//own
#include "config.h"
#include "recordlist.h"
#include "utilfactorydatafilename.h"

class ConfigManager
{
public:


    //! function to get the instance
    static ConfigManager* instance();

    //! function to destroy the instance
    static void drop();

    //! load will load the default config path
    static bool loadConfig();

    //! load config from file
    static bool loadConfig(QString path);

    //! write config to file
    static bool writeConfig(QString path, Config config);

    //! write config to file
    static bool writeRecordList(QString path, RecordList &recList);

    //! we will save the recordlist to default path
    static bool writeRecordList();


    //! @return the config which should be loaded
    //! @return nullptr if not loaded
    static Config* getConfig();

    static Config* conf;

    static RecordList* recList;


    //! reload the recordlist
    static bool reloadRecordList();



    //TODO:- mark this part as highl opencv dependend
    //TODO: find a non global version ???
    static bool m_loadedDistortionMatrixSet;
    static cv::Mat m_intrinsicsCameraMatrix;
    static cv::Mat m_distortionCoeff;
    //TODO maybe not needed anymore

private:

    static QJsonDocument loadJsonDocFromFile(QString file, bool &success);

    //! the one instance witch can exist
    static ConfigManager *m_instance;


    //! private
    ConfigManager();

    //! hide copy constructor
    ConfigManager(const ConfigManager&);

    //! override the copy constructor
    ConfigManager& operator=(const ConfigManager &); // hide assign op
    // we leave just the declarations, so the compiler will warn us
    // if we try to use those two functions by accident

};

#endif // CONFIGMANAGER_H
