#include "imagepreprocessutil.h"

Mat ImagePreProcessUtil::cropImage(Mat image,int north, int east, int south, int west)
{
    //qDebug()<<"cropImage image called";
    //qDebug()<<"s1: "<<image.size().height<< "; "<<image.size().width;

    int newHeight = image.rows - south -north ;
    int newWidth = image.cols - east-west;
    Rect myROI(west,north,newWidth ,newHeight);

    //qDebug()<<"s2: "<<newHeight<< "; "<<newWidth;

    // Crop the full image to that image contained by the rectangle myROI
    Mat croppedImage =  image(myROI);

    //work with the cropped one
    return  croppedImage.clone();
}

//Mat ImagePreProcessUtil::colorQuantifzation(Mat src, int amountClusters)
//{
//    Mat samples(src.rows * src.cols, 3, CV_32F);
//    for( int y = 0; y < src.rows; y++ )
//        for( int x = 0; x < src.cols; x++ )
//            for( int z = 0; z < 3; z++)
//                samples.at<float>(y + x*src.rows, z) = src.at<Vec3b>(y,x)[z];

//    Mat labels;
//    int attempts = 5;
//    Mat centers;
//    kmeans(samples, amountClusters, labels, TermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS, 10000, 0.0001), attempts, KMEANS_PP_CENTERS, centers );


//    Mat new_image = src.clone();

//    for( int y = 0; y < src.rows; y++ )
//        for( int x = 0; x < src.cols; x++ )
//        {
//            int cluster_idx = labels.at<int>(y + x*src.rows,0);
//            new_image.at<Vec3b>(y,x)[0] = centers.at<float>(cluster_idx, 0);
//            new_image.at<Vec3b>(y,x)[1] = centers.at<float>(cluster_idx, 1);
//            new_image.at<Vec3b>(y,x)[2] = centers.at<float>(cluster_idx, 2);
//        }
//    imshow( "clustered image", new_image );
//    waitKey(0);

//    return new_image;

//}

//Mat ImagePreProcessUtil::greyQuantifzation(Mat src, int amountClusters)
//{
//    Mat samples(src.rows * src.cols, 1, CV_32F);
//    for( int y = 0; y < src.rows; y++ )
//        for( int x = 0; x < src.cols; x++ )
//            samples.at<float>(y + x*src.rows, 0) = src.at<uchar>(y,x);

//    Mat labels;
//    int attempts = 5;
//    Mat centers;
//    kmeans(samples, amountClusters, labels, TermCriteria(CV_TERMCRIT_ITER|CV_TERMCRIT_EPS, 10000, 0.0001), attempts, KMEANS_PP_CENTERS, centers );


//    Mat imgClustered = src.clone();

//    for( int y = 0; y < src.rows; y++ )
//        for( int x = 0; x < src.cols; x++ )
//        {
//            int cluster_idx = labels.at<int>(y + x*src.rows,0);
//            imgClustered.at<uchar>(y,x)  = centers.at<float>(cluster_idx, 0);
//        }
//    //    imshow( "clustered image grey", new_image );
//    //    waitKey(0);

//    return imgClustered;
//}

Mat ImagePreProcessUtil::cropImageCircleAndResize(Mat image, int x, int y, int r, bool fillBlack)
{

    // Your Hough circle
    Vec3f circ(x,y,r); // Some dummy values for now

    // Draw the mask: white circle on black background
    Mat1b mask(image.size(), uchar(0));
    circle(mask, Point(circ[0], circ[1]), circ[2], Scalar(255), cv::FILLED);

    // Compute the bounding box
    Rect bbox(circ[0] - circ[2], circ[1] - circ[2], 2 * circ[2], 2 * circ[2]);

    Mat res;
    if(fillBlack)
    {// Create a black image
        res = Mat::zeros( image.size(), CV_8UC3 );
        res = Scalar(0,0,0); //fill with black color
    }else
        res = image.clone();

    // Copy only the image under the white circle to black image
    image.copyTo(res, mask);

    // Crop according to the roi
    res = res(bbox);

    return res;
}


Mat ImagePreProcessUtil::cropImageCircle(Mat image, int x, int y, int r)
{

    // Your Hough circle
    Vec3f circ(x,y,r);

    // Draw the mask: white circle on black background
    Mat1b mask(image.size(), uchar(0));
    circle(mask, Point(circ[0], circ[1]), circ[2], Scalar(255), cv::FILLED);

    //NO CROP NEEDED

    // Compute the bounding box
    //Rect bbox(circ[0] - circ[2], circ[1] - circ[2], 2 * circ[2], 2 * circ[2]);

    // Create a black image
    Mat res = Mat::zeros( image.size(), CV_8UC3 );
    res = Scalar(0,0,0); //fill with black color


    // Copy only the image under the white circle to black image
    image.copyTo(res, mask);

    // Crop according to the roi
    //res = res(bbox);

    return res;
}

bool ImagePreProcessUtil::fileExists(QString path) {
    QFileInfo check_file(path);
    // check if file exists and if yes: Is it really a file and no directory?
    if (check_file.exists() && check_file.isFile()) {
        return true;
    } else {
        return false;
    }
}

Mat ImagePreProcessUtil::makeCorrection(Mat input, QString cameraConfigFileName)
{
    Mat imgLenseCorrection = input.clone(); // TOOD?? not clone, because its may faster

    //TODO read in memory one time

    //one first call we try to load and mark this
    if(!ConfigManager::instance()->m_loadedDistortionMatrixSet)
    {
        QMutex mutex;

        mutex.lock();
        //we load the depends matrix
        if(fileExists(cameraConfigFileName))
        {

            FileStorage fs(cameraConfigFileName.toStdString(), FileStorage::READ);
            fs["camera_matrix"] >> ConfigManager::instance()->m_intrinsicsCameraMatrix;
            fs["distortion_coefficients"] >> ConfigManager::instance()->m_distortionCoeff;
            ConfigManager::instance()->m_loadedDistortionMatrixSet = true;
        }
        else
            qWarning()<<" !! error, will use distortion picture for analyse, file \"camera.yml\" not found in directory: "<<QDir::currentPath();
        qDebug()<<"ImagePreProcessUtil::makeCorrection matrix set loaded";
        mutex.unlock();

    }

    //correct lense distortion
    cv::undistort(input, imgLenseCorrection, ConfigManager::instance()->m_intrinsicsCameraMatrix, ConfigManager::instance()->m_distortionCoeff);

#ifdef  IPU_SHOW_STEPS
    //show
    imshow("undist Image", imgLenseCorrection);
    imwrite("ana_01_lenseCorrection.jpg", imgLenseCorrection);

    waitKey(0);
    cvDestroyWindow("undist Image");

#endif

    return imgLenseCorrection;

}
