#ifndef IMAGEPREPROCESSUTIL_H
#define IMAGEPREPROCESSUTIL_H


//qt
#include <QString>
#include <QDir>
#include <QDebug>
#include <QDirIterator>
#include <QTime>

//opencv
#include "opencv2/opencv.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"

//own
#include "configmanager.h"

//namespace
using namespace cv;
using namespace std;

//#define IPU_SHOW_STEPS

class ImagePreProcessUtil
{
public:

    //! returns a image, which is cutted on the sides north, south, west
    //! return roi of image which is cloned
    static Mat cropImage(Mat image,int north, int east, int south, int west);

    //! returns a image, which is resulting circle
    //! return roi of image which is cloned
    //! without resize the image
    //! will fill rest outer bound black
    static Mat cropImageCircle(Mat image, int x, int y, int r);

    //! returns a image, which is resulting circle
    //! return roi of image which is cloned
    //! will resize the image
    static Mat cropImageCircleAndResize(Mat image, int x, int y, int r, bool fillBlack);

    //! util to check if file is existing
    //! TODO: check we have this function double @see utilfactorydatafilename
    static bool fileExists(QString path);

//    //! test state
//    //!  //! @see http://docs.opencv.org/3.1.0/d1/d5c/tutorial_py_kmeans_opencv.html
//    //! @see http://stackoverflow.com/questions/10240912/input-matrix-to-opencv-kmeans-clustering/10242156#10242156
//    //! works with with 3 channel RGB pic
//    static Mat colorQuantifzation(Mat src, int amountClusters);

//    //! make color quantifization on a grey scale pic with knn algo:
//    //! @see http://docs.opencv.org/3.1.0/d1/d5c/tutorial_py_kmeans_opencv.html
//    //! @see http://stackoverflow.com/questions/10240912/input-matrix-to-opencv-kmeans-clustering/10242156#10242156
//    //!  @warning not working for color pics, we work with CV_8UC1 grayscale
//    static Mat greyQuantifzation(Mat src, int amountClusters);

    //! create Picture with is correct with image lense correction
    //! @see http://docs.opencv.org/trunk/dc/dbb/tutorial_py_calibration.html
    static Mat makeCorrection(Mat input, QString cameraConfigFileName );


};

#endif // IMAGEPREPROCESSUTIL_H
