#include "imageprocesstask.h"

cv::Mat ImageProcessTask::image() const
{
    return m_image;
}

void ImageProcessTask::setImage(const cv::Mat &image)
{
    m_image = image;
}

quint64 ImageProcessTask::imageNumber() const
{
    return m_imageNumber;
}

void ImageProcessTask::setImageNumber(const quint64 &imageNumber)
{
    m_imageNumber = imageNumber;
}

QString ImageProcessTask::path() const
{
    return m_path;
}

void ImageProcessTask::setPath(const QString &path)
{
    m_path = path;
}

void ImageProcessTask::setDateTime(QTime t, QDateTime dt)
{
    m_time = t;
    m_dateTime = dt;
}

quint64 ImageProcessTask::second() const
{
    return m_second;
}

void ImageProcessTask::setSecond(const quint64 &second)
{
    m_second = second;
}

quint64 ImageProcessTask::milliSecond() const
{
    return m_milliSecond;
}

void ImageProcessTask::setMilliSecond(const quint64 &milliSecond)
{
    m_milliSecond = milliSecond;
}

StartMoment ImageProcessTask::startMoment() const
{
    return m_startMoment;
}

void ImageProcessTask::setStartMoment(const StartMoment &startMoment)
{
    m_startMoment = startMoment;
}

void ImageProcessTask::run()
{
    QTime myTimer;
    myTimer.start();
    
    //qDebug() << "process image"<<m_imageNumber<<" on frame thread" << QThread::currentThread();
    
    QString pathStr = m_path;
    //QString timeTableName = "tt.txt";
    //create path if not exstings
    if(!QDir(pathStr).exists())
    {   qDebug()<<"will create path "<< pathStr;
        QDir().mkpath(pathStr);
    }

    //what should format should we use ?
    //https://ieeexplore.ieee.org/document/7823671/
    //https://link.springer.com/article/10.1007/s11760-010-0191-7


    //maybe png as loseless format with moderate compression ?
    //test on server showed: "good" compression takes to long for a life system

    //perhaps stay jpeg which high quality



    //qDebug()<<m_imageNumber<<"\t---------------------------";
#ifdef PROCESS_STEP_CONVERT_RGB_TO_BGR
    //a)
    //we need to convert rgb to bgr
    //cv::Mat imgBGR;
    cv::cvtColor(m_image,m_image, cv::COLOR_RGB2BGR) ;
#endif
#ifdef PRINT_TIME_USED_FOR_OPERATION
    QList<int> m_timesList;

    m_timesList.append( myTimer.restart());
    qDebug()<<"step a)\t"<< m_timesList.last()<< "ms convert color rgb to rgr ";
#endif

#ifdef PROCESS_STEP_MAKE_LENSE_CORRECTION
    //b) we make lense correction
    m_image = ImagePreProcessUtil::makeCorrection(m_image,ConfigManager::instance()->conf->getFileLenseCorrection());
#endif
#ifdef PRINT_TIME_USED_FOR_OPERATION
    m_timesList.append(  myTimer.restart() );
    qDebug()<<"step b)\t"<< m_timesList.last()<< "ms make lense correction";
#endif

#ifdef PROCESS_STEP_CUT_ROI

    //c) we cut everything we need
    //we will cut a circle for the arena in the picture
    m_image =  ImagePreProcessUtil::cropImageCircle(m_image,1000,630,650);
#endif
#ifdef PRINT_TIME_USED_FOR_OPERATION
    m_timesList.append( myTimer.restart() );
    qDebug()<<"step c)\t"<< m_timesList.last()<< "ms cut out ROI";
#endif


    //use the opencv method


    //try some compression //@see https://docs.opencv.org/2.4/modules/highgui/doc/reading_and_writing_images_and_video.html#imwrite
    //jp2 -> no effect

#ifdef PROCESS_STEP_WRITE_COMPRESSED

    QString numberStr = QString::number(m_imageNumber).rightJustified(10, '0');

    QString formatStr;
    std::vector<int> compression_params;

#ifdef PROCESS_OUTPUT_JPG_QUALITY_100 //TODO this is not the best case
    formatStr = ".jpg";

    //no progressive, because https://ux.stackexchange.com/questions/81073/progressive-vs-optimized-jpegs
    compression_params.push_back(cv::IMWRITE_JPEG_QUALITY);
    compression_params.push_back(100);//we use the best compression, full pic will end up with 1,2 MB
    compression_params.push_back(cv::IMWRITE_JPEG_OPTIMIZE);
    compression_params.push_back(1);
    compression_params.push_back(cv::IMWRITE_JPEG_RST_INTERVAL);
    compression_params.push_back(4);
    //    //not used, because what is the effect ?
    //    compression_params.push_back(cv::IMWRITE_JPEG_CHROMA_QUALITY);
    //    compression_params.push_back(100);
    //    compression_params.push_back(cv::IMWRITE_JPEG_LUMA_QUALITY);
    //    compression_params.push_back(100);
#endif

#ifdef PROCESS_OUTPUT_PNG
    formatStr = ".png";
    //we also different strategies
    //@sse https://github.com/opencv/opencv/issues/6729
    //9-> takes to long but looks good
    compression_params.push_back(cv::IMWRITE_PNG_COMPRESSION);
    compression_params.push_back(1);
    //    compression_params.push_back(cv::IMWRITE_PNG_STRATEGY);
    //    compression_params.push_back(cv::IMWRITE_PNG_STRATEGY_FILTERED);

#endif
#ifdef PROCESS_OUTPUT_PNG_LOSELESS_FAST_WRITE_HIGH_SPACE
    formatStr = ".png";
    compression_params.push_back(cv::IMWRITE_PNG_COMPRESSION);
    compression_params.push_back(0);

    compression_params.push_back(cv::IMWRITE_PNG_STRATEGY);
    compression_params.push_back(cv::IMWRITE_PNG_STRATEGY_DEFAULT);

#endif

#ifdef PROCESS_OUTPUT_PPM
    formatStr = ".ppm";
    compression_params.push_back(cv::IMWRITE_PXM_BINARY);
    compression_params.push_back(1);

#endif
    QString fileName = numberStr + formatStr;
    QString allStr = pathStr + fileName;


    //webp
    //    compression_params.push_back(cv::IMWRITE_WEBP_QUALITY);
    //    compression_params.push_back(200);

    cv::imwrite(allStr.toStdString(),m_image,compression_params);

#endif
#ifdef PRINT_TIME_USED_FOR_OPERATION
    m_timesList.append(  myTimer.restart()) ;
    qDebug()<<"step d)\t"<< m_timesList.last()<< " ms write compressed";
#endif

#ifdef PROCESS_STEP_WRITE_EASY_TT_FILE

    QString fileNameTT = numberStr + ".tt";
    QString fileOutTT = pathStr + fileNameTT;

    //here we calc the number format since linux epoch
    QPair<quint64,quint64> timeStamp = m_startMoment.getDiff(m_second,m_milliSecond);


    QFile outFile(fileOutTT);
    if(outFile.open(QIODevice::WriteOnly|QIODevice::Truncate))
    {
        QString timeStr = QString("%1;%2;")
                .arg(timeStamp.first)
                .arg(timeStamp.second);
        QTextStream ts(&outFile);
        ts << timeStr << numberStr <<endl;
        //qDebug()<<timeStr << numberStr;
        //qDebug()<<m_imageNumber<<"|"<<timeStr<<"|"<<numberStr;
    }
    else
        qWarning()<<"error during write time table file, information lost";

    outFile.close();
#endif

    //This old one is bad because the processes will block each other
#ifdef PROCESS_WRITE_TIME_TABLE_FILE


    //write timeTableimgCircled
    QFile outFile(pathStr +timeTableName);
    if(!outFile.open(QIODevice::WriteOnly | QIODevice::Append))
    {
        fprintf(stderr, "error while open time table file !! \n");
    }
    else
    {
        QString timeStr = QString("%1;%2;")
                .arg(m_dateTime.toTime_t())
                .arg(m_time.msec());

        QTextStream ts(&outFile);
        ts << timeStr << fileName <<endl;
        //say nothing on success
        //std::stringstream strMsg;
        //strMsg << "write file .. okay"<<allStr.toStdString() ;
        //Log(strMsg.str() );
    }

    outFile.close();//close after reading
#endif

    int allTimes = 0;
#ifdef PRINT_TIME_USED_FOR_OPERATION
    foreach (int t, m_timesList) {
        allTimes+=t;
    }
#else
    allTimes = myTimer.elapsed();
#endif
    //qDebug()<<m_imageNumber<<"frame\t-\t"<<allTimes<<"ms\t"<< (double)( (double) 1000 / (double)allTimes) << " fps\t" << QThread::currentThread()<<" - "<< QThreadPool::globalInstance()->activeThreadCount();

    //TODO maybe we implement a thread count

    //here we try to free and release everything
    m_image.release();
}
