#ifndef IMAGEPROCESSTASK_H
#define IMAGEPROCESSTASK_H

//qt
#include <QRunnable>
#include <QDebug>
#include <QThreadPool>
#include <QString>
#include <QFile>
#include <QDir>
#include <QTimer>
#include <QDateTime>

//opencv
#include "opencv2/core/core.hpp"
#include "opencv2/opencv.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"

//own
#include "imagepreprocessutil.h"
#include "configmanager.h"
#include "startmoment.h"

////libarchive
//#include <archive.h>
//#include <archive_entry.h>

#define PROCESS_STEP_CONVERT_RGB_TO_BGR
//#define PROCESS_STEP_MAKE_LENSE_CORRECTION
//#define PROCESS_STEP_CUT_ROI
#define PROCESS_STEP_WRITE_COMPRESSED
#define PROCESS_STEP_WRITE_EASY_TT_FILE


#define PROCESS_OUTPUT_JPG_QUALITY_100
//#define PROCESS_OUTPUT_PNG
//#define PROCESS_OUTPUT_PNG_LOSELESS_FAST_WRITE_HIGH_SPACE
//#define PROCESS_OUTPUT_PPM

//#define PROCESS_WRITE_TIME_TABLE_FILE


//#define PRINT_TIME_USED_FOR_OPERATION

class ImageProcessTask : public QRunnable
{
public:
    //! the method which performs the
    void run() override;

    cv::Mat image() const;
    void setImage(const cv::Mat &image);

    quint64 imageNumber() const;
    void setImageNumber(const quint64 &imageNumber);

    QString path() const;
    void setPath(const QString &path);

    void setDateTime(QTime t,QDateTime dt);

    quint64 second() const;
    void setSecond(const quint64 &second);

    quint64 milliSecond() const;
    void setMilliSecond(const quint64 &milliSecond);

    StartMoment startMoment() const;
    void setStartMoment(const StartMoment &startMoment);

private:
    cv::Mat m_image;
    quint64 m_imageNumber;
    QString m_path;
    quint64 m_second;
    quint64 m_milliSecond;
    StartMoment m_startMoment;



    QTime m_time = QTime::currentTime();
    QDateTime m_dateTime =  QDateTime::currentDateTime();
};

#endif // IMAGEPROCESSTASK_H
