//qt
#include <QCoreApplication>
#include <QTcpServer>
#include <QTimer>
#include <QCommandLineParser>

//std
#include <iostream>
#include <stdio.h>

//open cv
//#include "opencv2/core/core.hpp"
#include "opencv2/opencv.hpp"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"

#include "myhttpserver.h"
#include "configmanager.h"
#include "utilfactorydatafilename.h"
#include "asyncorder.h"
#include "recordlist.h"


using namespace cv;
using namespace std;

using AVT::VmbAPI::CameraPtrVector;

void mySleep(int sec);

int main( int argc, char *argv[] )
{

    QCoreApplication a( argc, argv );


    qDebug()<<"main thread id:"<<QThread::currentThreadId();


    QCommandLineParser parser;
    parser.setApplicationDescription("camera recorder");
    parser.addHelpOption();
    parser.addVersionOption();
    parser.addPositionalArgument("file", QCoreApplication::translate("main", "please specifi a folder"));

    // Process the actual command line arguments given by the user
    parser.process(a);

    //test if we have enough args
    const QStringList args = parser.positionalArguments();

    if(args.size()!=1)
    {
        qWarning()<<"Please enter arguments\ne.g. ./foo cam1 (or cam2)";
        return -1;
    }

    QString choosenCam= args.at(0);
    int camSelect = -1;

    if(QString::compare(choosenCam,"cam1", Qt::CaseInsensitive)  == 0)
    {
        camSelect = 1;
    }
    else if (QString::compare(choosenCam,"cam2", Qt::CaseInsensitive)  == 0)
    {
        camSelect = 2;
    }
    else
    {
        qWarning()<<"error wrong argument :"<< args.at(0);
        return -1;
    }

    //    //test uid
    //    for(int i=0;i<10;i++)
    //        UtilFactoryDataFileName::getUID();
    //TODO change the timer
    //    auto begin = std::chrono::high_resolution_clock::now();
    //    // code to benchmark
    //    auto end = std::chrono::high_resolution_clock::now();
    //    std::cout << std::chrono::duration_cast<std::chrono::nanoseconds>(end-begin).count() << "ns" << std::endl;

    //calc hash test
    //    qDebug()<<"hash: "<< UtilFactoryDataFileName::getHash(".")<< " for folder: .";
    //    qDebug()<<"hash: "<< UtilFactoryDataFileName::getHash("/homes/tb55xemi/records-tracking-sys/klausine02/")<< " for folder: /homes/tb55xemi/records-tracking-sys/klausine02/";

    //write the config file for the first time **************************

    //test the config write option

//        Config configObject;
//        configObject.setStoragePath("/tmp/foo/");
//        //configObject.setFileLenseCorrection("./camera.yml");
//        configObject.setMinDiskSpaceNeeded(5);
//        configObject.setRecordDurationInSec(3600);

//        QList<ConfCamera> mCamList;

//        ConfCamera cam1;
//        cam1.setCamCenterX(100);
//        cam1.setCamCenterY(200);
//        cam1.setCamRadius(300);
//        cam1.setPortAdress(8080);
//        cam1.setCamName("cam1");
//        mCamList.append(cam1);

//        ConfCamera cam2;
//        cam2.setCamCenterX(400);
//        cam2.setCamCenterY(500);
//        cam2.setCamRadius(600);
//        cam2.setPortAdress(8081);
//        cam2.setCamName("cam2");
//        mCamList.append(cam2);

//        configObject.setCamList(mCamList);

//        if(!ConfigManager::instance()->writeConfig("./",configObject))
//            qWarning()<<"error during write config file";

    //    //test recordlist ********
    //    RecordList recList;

    //    QList<RecordInfo> recs;
    //    RecordInfo recA;// = new RecordInfo();
    //    recA.setAmountLostFrames(23);
    //    recA.setUID(UtilFactoryDataFileName::getUID());
    //    recA.setState(RecordInfo::STATE_RECORDED_WITH_SUCCESS);


    //    recs.append(recA);
    //    recList.setRecordList(recs);

    //    if(!ConfigManager::instance()->writeRecordList("./",recList))
    //        qWarning()<<"error during write config file";

    //global conf read

    if(!ConfigManager::instance()->loadConfig())
        qWarning()<<"error during load config";
    else
        qDebug()<<"load config: done";

    //test access
    //qDebug()<<ConfigManager::instance()->conf->toString();

    MyHTTPserver *server = new MyHTTPserver();

    QThread *recordThread = new QThread();
    AsynCorder *recorder = new AsynCorder();
    recorder->moveToThread(recordThread);

    //we should register not none enum type
    qRegisterMetaType<AsynCorder::STATE>("AsynCorder::STATE");

    //connect the state exchange of the server
    QObject::connect(recorder, SIGNAL(stateChange(AsynCorder::STATE)), server, SLOT(cameraRecorderStateChanged(AsynCorder::STATE)));

    //start the thread
    QObject::connect(recordThread, SIGNAL (started()), recorder, SLOT (process()));
    recordThread->start(QThread::TimeCriticalPriority);


    qDebug()<<"wait for cams to be found with vimba detection";
    mySleep(10);
    qDebug()<<"wait done"; 
    QStringList myList = recorder->getCamList();
    foreach (QString s, myList) {
        qDebug()<<s;
    }
    if(myList.isEmpty())
    {
	qWarning()<<"error 98383: no camera found by vimba disovery service, will exit here";
	return -1;
    }

    //QThreadPool::globalInstance()->setMaxThreadCount(2);

    QList<ConfCamera> camConfList = ConfigManager::instance()->conf->getCamList();

    if(camSelect==1)
    {

        //TODO we should also match the config with the detected ones

        //choose the first one
        if(!myList.isEmpty())
        {
            qDebug()<<"recorder1:";
            //        recorder1->printThreadID();
            QString camID = myList.at(0);
            QString camName = camConfList.at(0).getCamName();

            //doube check the uid of the cam
            if(QString::compare(camID,camName,Qt::CaseInsensitive) == 0)
            {
                //we have a match
                qDebug()<<"...."<<camID;
                recorder->chooseCamToRecord(camID);
            }
            else
            {
                qCritical()<<"error during match camerID:" << camID<<" will abort";
                return -1;
            }

        }
        if(!camConfList.isEmpty())
        {
            server->setConfig(camConfList.at(0));
            if(!server->initServer())
            {
                qWarning()<<"error during start server, will abort execution";
                return -1;
            }
        }
        //    QObject::connect(server,SIGNAL(emitStartCam1()),recorder1,SLOT(startRecord()));
        //    QObject::connect(server,SIGNAL(emitStoppCam1()),recorder1,SLOT(stopRecord()));

    }
    if(camSelect ==2)
    {

        if(myList.size()==2)
        {

            qDebug()<<"recorder2:";
            //        recorder2->printThreadID();
            QString camID = myList.at(1);
            QString camName = camConfList.at(1).getCamName();


            //doube check the uid of the cam
            if(QString::compare(camID,camName, Qt::CaseInsensitive)==0)
            {
                //we have a match
                qDebug()<<"...."<<camID;
                recorder->chooseCamToRecord(camID);
            }
            else
            {
                qCritical()<<"error during match camerID:" << camID<<" will abort";
                return -1;
            }

        }

        if(camConfList.size()==2)
        {
            server->setConfig(camConfList.at(1));
            if(!server->initServer())
            {
                qWarning()<<"error during start server, will abort execution";
                return -1;
            }
        }

        //    QObject::connect(server,SIGNAL(emitStartCam2()),recorder2,SLOT(startRecord()));
        //    QObject::connect(server,SIGNAL(emitStoppCam2()),recorder2,SLOT(stopRecord()));

    }

    //triggered by the server app
    QObject::connect(server,SIGNAL(emitStoppCam()),recorder,SLOT(stopRecord()));

    //conncet the timout out with start
    QObject::connect(server,SIGNAL(emitStartCam()),recorder,SLOT(startDelay()));

    //*************** timing
//    QTimer *timerStart = new QTimer(QCoreApplication::instance());
//    //timer->setSingleShot(true);
//    timerStart->setInterval(1000 * 60 * 2 ); // 1 minutes
//    timerStart->start();

//    QTimer *timerStop = new QTimer(QCoreApplication::instance());
//    //timer->setSingleShot(true);
//    timerStop->setInterval(1000 * 60 * 1 ); // 1 minutes
//    //timer->setInterval(1000 * 60 * 60); // one hour
//    timerStop->start();

//    qDebug()<<"1";
//    recorder->startRecord();

//    QObject::connect(timerStop,SIGNAL(timeout()),recorder,SLOT(stopRecord()));
//    QObject::connect(timerStart,SIGNAL(timeout()),recorder,SLOT(startRecord()));

//    int cycleAmount = 4;

//    for(int i=1;i<cycleAmount;i++)
//    {
//        mySleep(1000 * 60 * 2 ); //wait in main thread 2 min
//        qDebug()<<cycleAmount;
//        //again
//        //timer->start();
//        recorder->startRecord();
//    }
    //AsynchronousGrab w;
    //w.show();
    //

    return a.exec();
}

void mySleep(int sec)
{
    QTime dieTime= QTime::currentTime().addSecs(sec);
    while (QTime::currentTime() < dieTime)
        QCoreApplication::processEvents(QEventLoop::AllEvents, 100);

}
