#include "myhttpserver.h"

MyHTTPserver::MyHTTPserver(QObject *parent) : QObject(parent)
{


}

void MyHTTPserver::cameraRecorderStateChanged(AsynCorder::STATE s)
{
    m_camState = s;
    qDebug()<<"server state changed: "<< AsynCorder::stateToString(s);

}

bool MyHTTPserver::initServer()
{
    m_camState = AsynCorder::STATE_UNKNOWN;

    bool success = false;
    server = new QTcpServer(this);
    // waiting for the web brower to make contact,this will emit signal
    connect(server, SIGNAL(newConnection()),this, SLOT(myConnection()));
    if(!server->listen(QHostAddress::Any,m_config.getPortAdress()))
    {
        qDebug()<< "\nWeb server      could not start";
        success = false;
    }
    else
    {
        qDebug()<<"\nWeb server is waiting for a connection on port:"<<m_config.getPortAdress();
        success = true;
    }
    return success;
}

void MyHTTPserver::myConnection()
{
    socket = server->nextPendingConnection();
    while(!(socket->waitForReadyRead(100)));  //waiting for data to be read from web browser

    char webBrowerRXData[1000];
    int sv=socket->read(webBrowerRXData,1000);
    //qDebug()<<"\nreading web browser data=\n";
    //convert to string
    QString s = QString::fromLatin1(webBrowerRXData,sv);
    //    for(int i=0;i<sv;i++)qDebug()<<webBrowerRXData[i];
    //qDebug()<<s;

    //we parse our rest mapping things

    QHttpPart textPart;
    textPart.setHeader(QNetworkRequest::ContentTypeHeader, QVariant("application/json"));
    //textPart.setHeader(QNetworkRequest::
    textPart.setRawHeader(QByteArray("HTTP/1.1 200"),QByteArray("OK"));
    textPart.setBody("{}");

    socket->write("HTTP/1.1 200 OK\r\n");       // \r needs to be before \n
    socket->write("Server: foo42\r\n");
    socket->write("Access-Control-Allow-Origin: * \r\n");
    socket->write("Content-Type: application/json\r\n");

    socket->write("Connection: close\r\n");
    //socket->write("Refresh: 1\r\n");     //refreshes web browser     every second. Require two \r\n.

    // blank line seperates the header from the body
    //@see https://www.ntu.edu.sg/home/ehchua/programming/webprogramming/HTTP_Basics.html
    socket->write("\r\n");


    QJsonObject jsonResponse;
    jsonResponse["result"] = "none";

    /*
    if(s.contains("cam", Qt::CaseInsensitive))
    {
        //we provide a json with the cameras
        qDebug()<<"space";
        jsonResponse["result"] = "ok";
        jsonResponse["foo"] = "test";
    }
    */

    if(s.contains("space", Qt::CaseInsensitive))
    {
        //we provide a json with the cameras
        qDebug()<<"space";
        jsonResponse["result"] = "ok";
        jsonResponse["spaceLeftInGB"] = UtilFactoryDataFileName::getAmountGBLeft();

    }

    if(s.contains("startrecord", Qt::CaseInsensitive))
    {
        if(s.contains("cam", Qt::CaseInsensitive))
        {
            //we provide a json with the cameras
            emit emitStartCam();


            qDebug()<<"wait 10 min delay cam";

            //we will force a wait of 1500 ms, needed to be sure the signal arrives at the socket
            socket->waitForReadyRead(1500);
            qDebug()<<"state: "+ AsynCorder::stateToString(m_camState);
            jsonResponse["state"] = AsynCorder::stateToString(m_camState);


        }

    }
    if(s.contains("stoprecord", Qt::CaseInsensitive))
    {
        if(s.contains("cam", Qt::CaseInsensitive))
        {
            //we provide a json with the cameras

            qDebug()<<"stop cam";
            jsonResponse["cam"] = "cam";
            jsonResponse["result"] = "ok";

            emit emitStoppCam();
        }
    }

    if(s.contains("state", Qt::CaseInsensitive))
    {
        if(s.contains("cam", Qt::CaseInsensitive))
        {
            //we provide a json with the cameras

            qDebug()<<"state: "+ AsynCorder::stateToString(m_camState);
            jsonResponse["state"] = AsynCorder::stateToString(m_camState);
            //depending on the state we add some more infos
            if(m_camState == AsynCorder::STATE_RECORDING)
            {
                RecordInfo info = ConfigManager::instance()->recList->getLastRecordInfo();
                jsonResponse["cam"] = info.getCamID();
                jsonResponse["result"] = "ok";
                jsonResponse["recordUID"] = info.getUID();
            }
            else
                jsonResponse["recordUID"] = "0";
        }
    }

    //convert our string to something we could send

    QJsonDocument doc(jsonResponse);
    QString strJson(doc.toJson(QJsonDocument::Compact));
    strJson.append("\r\n");
    //qDebug()<<strJson;
    socket->write(strJson.toUtf8());

    socket->flush();
    connect(socket, SIGNAL(disconnected()),socket, SLOT(deleteLater()));
    socket->disconnectFromHost();
}

void MyHTTPserver::setConfig(const ConfCamera &config)
{
    m_config = config;
}

ConfCamera MyHTTPserver::getConfig() const
{
    return m_config;
}

MyHTTPserver::~MyHTTPserver()
{
    socket->close();
}
