#include "utilfactorydatafilename.h"

//************ singelton stuff **** START

//must be for declerate the singelton
UtilFactoryDataFileName* UtilFactoryDataFileName::m_instance = 0;


UtilFactoryDataFileName::UtilFactoryDataFileName()
{
}
UtilFactoryDataFileName* UtilFactoryDataFileName::instance()
{

    static QMutex mutex;
    if (!m_instance)
    {
        mutex.lock();

        if (!m_instance)
            m_instance = new UtilFactoryDataFileName();

        mutex.unlock();
    }

    return m_instance;
}

void UtilFactoryDataFileName::drop()
{
    static QMutex mutex;
    mutex.lock();
    delete m_instance;
    m_instance = 0;
    mutex.unlock();
}
//************ singelton stuff **** END

QString UtilFactoryDataFileName::getDefaultPathConfigFile()
{
    return QString("./");
}

QString UtilFactoryDataFileName::getPathStringOfRecord(RecordInfo rec)
{
    //get the path
    QString s( ConfigManager::instance()->conf->storagePath());
    //we add a prefix
    s.append("/rec");
    //and the uid with leading zeros
    s.append(QString::number(rec.getUID()).rightJustified(UTIL_UID_DIGITS, '0'));
    s.append("/data/");
    return s;
}

quint64 UtilFactoryDataFileName::getUID()
{
    //get random number uid for the pic output
    std::random_device rd;
    std::mt19937 gen(rd());
    std::uniform_int_distribution<unsigned long long> dis(0,(int) pow((int)10, (int) UTIL_UID_DIGITS));
    quint64 foo = dis(gen);
    qDebug()<<"produced UID:"<< foo;
    return foo;
}

bool UtilFactoryDataFileName::fileExists(QString file)
{
    QFileInfo check_file(file);
    // check if file exists and if yes: Is it really a file and no directory?
    if (check_file.exists() && check_file.isFile()) {
        return true;
    } else {
        return false;
    }
}

bool UtilFactoryDataFileName::pathExists(QString path)
{
    return QDir(path).exists();
}

bool UtilFactoryDataFileName::pathExistsAndIsNotEmpty(QString path)
{
    QDir fooDir(path);
    bool pathExist = fooDir.exists();
    bool pathExistAndIsNotEmpty = pathExist;

    if(pathExist)
    {
        //
        QFileInfoList list = fooDir.entryInfoList();


        if(list.size() == 2)
            pathExistAndIsNotEmpty = false; //exists and is empty
        else
            pathExistAndIsNotEmpty = true; //exists and is not empty

    }

    return pathExistAndIsNotEmpty;
}

void UtilFactoryDataFileName::createDir(QString dir)
{
    if(!(QDir(dir).exists()))
    {
        bool succes  =  QDir().mkpath(dir);
        if(!succes)
            qWarning()<<"error during create dir "<<dir;
    }
}

QString UtilFactoryDataFileName::getHash(QString filePath)
{
    // create a fileInfo from the top-level directory
    QFileInfo fileInfo(filePath);
    QString hash;
    // Choose an arbitrary hash, say Sha1
    QCryptographicHash cryptHash(QCryptographicHash::Sha1);
    // add all files to the hash
    addToHash(fileInfo, cryptHash);
    // get a printable version of the hash
    hash = cryptHash.result().toHex();

    return hash;
}

void  UtilFactoryDataFileName::addToHash(const QFileInfo& fileInf, QCryptographicHash& cryptHash)
{
    QDir directory(fileInf.absoluteFilePath());
    directory.setFilter(QDir::NoDotAndDotDot | QDir::AllDirs | QDir::Files);
    QFileInfoList fileInfoList = directory.entryInfoList();

    foreach(QFileInfo info, fileInfoList)
    {
        if(info.isDir())
        {
            // recurse through all directories
            addToHash(info, cryptHash);
            continue;
        }

        // add all file contents to the hash
        if(info.isFile())
        {
            QFile file(info.absoluteFilePath());
            if(!file.open(QIODevice::ReadOnly))
            {
                // failed to open file, so skip
                continue;
            }
            cryptHash.addData(&file);
            file.close();
        }
    }
}


QString UtilFactoryDataFileName::getDatePattern()
{
    return QDateTime::currentDateTime().toString("yyyy-MM-dd");
}

QString UtilFactoryDataFileName::getTimePattern()
{
    return QTime::currentTime().toString("HH-mm-ss-zzz-t");
}

int UtilFactoryDataFileName::getAmountGBLeft()
{
    QStorageInfo storage = QStorageInfo::root();

    qDebug() << storage.rootPath();
    if (storage.isReadOnly())
        qDebug() << "isReadOnly:" << storage.isReadOnly();

    quint64 bytesLeft = storage.bytesAvailable();
    //    quint64 mBytesLeft = bytesLeft/BYTE_PER_MEGABYTE;
    quint64 gBytesLeft = bytesLeft/BYTE_PER_GIGABYTE;
    //    qDebug()<<"MB left"<<mBytesLeft;
    //    qDebug()<<"GB left"<<gBytesLeft;


    return (int) gBytesLeft;

    //    qDebug() << "name:" << storage.name();
    //    qDebug() << "fileSystemType:" << storage.fileSystemType();
    //    qDebug() << "size:" << storage.bytesTotal()/1000/1000 << "MB";
    //    qDebug() << "availableSize:" << storage.bytesAvailable()/1000/1000 << "MB";

}

bool UtilFactoryDataFileName::enoughtDiskSpaceLeft()
{
    int gBytesLeft = getAmountGBLeft();

    if(gBytesLeft>(quint64)ConfigManager::instance()->conf->minDiskSpaceNeeded())return true;

    return false;

}



