#ifndef UTILFACTORYDATAFILENAME_H
#define UTILFACTORYDATAFILENAME_H

//TODO: make enoughtDiskSpaceLeft for unix !!! see  enoughtDiskSpaceLeft
//TODO: test if the path exist

//std
#include <iostream>
#include <stdio.h>
#include <random>

// qt
#include <QMutex>
#include <QDateTime>
#include <QDate>
#include <QDebug>
#include <QFile>
#include <QStringList>
#include <QFileInfo>
#include <QStorageInfo>
#include <QCryptographicHash>

//own
#include "configmanager.h"
#include "recordinfo.h"

#define UTIL_FACTORY_DATA_FILE_NAME_DEBUG_MODE 0
#define UTIL_UID_DIGITS 8
#define BYTE_PER_GIGABYTE 1073741824 //GiB
#define BYTE_PER_MEGABYTE 1048576    //MiB
#define BYTE_PER_KILOBYTE 1048576    //KiB

//! util to produce good file names
//! is a singelton
class UtilFactoryDataFileName
{
public:

    //! function to get the instance
    static UtilFactoryDataFileName* instance();

    //! function to destroy the instance
    static void drop();

    static QString getPathStringOfRecord(RecordInfo rec);

    //! test if file is existing, return true if yes
    static bool fileExists(QString file);

    //! check diskspace
    //! @return true if over HARD_DISK_SPACE_NEEDED_IN_GiB
    static bool enoughtDiskSpaceLeft();

    //! return the amount of gb left on the device
    static int getAmountGBLeft();

    //! get the datepattern for fileNames
    static QString getDatePattern();

    //! get readable time
    static QString getTimePattern();

    static quint64 getUID();

    //!
    static QString getDefaultPathConfigFile();

    //! first check if dir exist, when not i will be created
    static void createDir(QString dir);

    //! return true if path is existing
    static bool pathExists(QString path);

    static bool pathExistsAndIsNotEmpty(QString path);

    //! return a hash for a specific folder
    static QString getHash(QString path);

private:
    //! the one instance witch can exist
    static UtilFactoryDataFileName *m_instance;

    //! private
    UtilFactoryDataFileName();

    //! hide copy constructor
    UtilFactoryDataFileName(const UtilFactoryDataFileName&);

    //! override the copy constructor
    UtilFactoryDataFileName& operator=(const UtilFactoryDataFileName &); // hide assign op
                         // we leave just the declarations, so the compiler will warn us
                         // if we try to use those two functions by accident


    //! recursive function for all files
    //! @see dissusion https://stackoverflow.com/questions/19863970/creating-a-hash-value-of-a-directory-in-qt
    static void addToHash(const QFileInfo& fileInf, QCryptographicHash& cryptHash);

};

#endif // UTILFACTORYDATAFILENAME_H
