/*
1. generate pattern
	-> checkerboard, circle or random
2. take at least 10 photos of pattern
	-> different angles and positions
3. calculate parameters
	extract corners of pattern image
		-> imgpoints
	corresponding 3d points
		-> objpoints
	imgSize
4. calibration
	-> calculate camera_matrix and distortion_coefficients
	-> store in xml

OPT
x. test accuracy with projectPoints (should be as close to 0 as possible)

https://opencv24-python-tutorials.readthedocs.io/_/downloads/en/stable/pdf/#subsection.1.7.1
*/


#include <iostream>
#include <opencv2/opencv.hpp>
#include <opencv2/core.hpp>
#include <opencv2/ccalib.hpp>
#include <opencv2/calib3d.hpp>
#include <opencv2/ccalib/randpattern.hpp>

// using namespace cv;
using namespace std;

void gen_rand_pattern(const int width, const int height)

const std::string outFileName = "pattern_for_camera_calibration.png";
const cv::string keys =
"{help h usage ? | | camera calibration help msg }"
"{w width | 300 | pattern width }"
"{h height | 300 | pattern height }"
"{g generate | | generate, show and save a calibration pattern }"
"{c calibrate | | calibrate camera }"
;

int main(int argc, char const *argv[])
{
	cv::CommandLineParser parser( argc, argv, keys );

	if (!parser.check())
	{
		parser.printErrors();
		return 0;
	}

	if (parser.has("g"))
	{
		cout << "GENERATE" << endl;
		generate_pattern();
	}
	else if (parser.has("d"))
	{
		cout << "DETECT" << endl;
		calibrate();
	}


	return 0;
}

void generate_pattern()
{
	int width = parser.get<int>("width");
	int height = parser.get<int>("height");

	std::cout << "width: " << width << std::endl;
	std::cout << "height: " << height << std::endl;

	cv::randpattern::RandomPatternGenerator generator(width, height);
	generator.generatePattern();

	cv::Mat pattern = generator.getPattern();

	cv::imshow("random Pattern", pattern);
	int k = waitKey(0); // Wait for a keystroke in the window
	cv::imwrite(outFileName, pattern);
	std::cout << "Saved pattern to " << outFileName <<
	"Now print pattern, take photos of it. Then run 'calibrate' on those photos" << std::endl;
}

void calibrate()
{
	//calc parameters
	//patternWidth+patternHeight are physical pattern width and height with some user defined unit
	patternWidth = width;
	patternHeight = height;
	std::vector vecImg; //vector of calibration images

	int nminiMatch = 20; // # of minimal matches to NOT abort image
	cv::randpattern::RandomPatternCornerFinder finder(patternWidth, patternHeight, nMiniMatches);
	finder.loadPattern(pattern);

	finder.computeObjectImagePoints(vecImg);
	std::vector<cv::Mat> imagePoints = finder.getImagePoints();	//2d Vec2f
	std::vector<cv::Mat> objectPoints = finder.getObjectPoints(); //3d Vec3f
	//size


	//calibrate
	cv::calibrateCamera(InputArrayOfArrays objectPoints, InputArrayOfArrays imagePoints, Size imageSize, InputOutputArray cameraMatrix, InputOutputArray distCoeffs, OutputArrayOfArrays rvecs, OutputArrayOfArrays tvecs, OutputArray stdDeviationsIntrinsics, OutputArray stdDeviationsExtrinsics, OutputArray perViewErrors)
}
